@@ -157,17 +157,17 @@ enaml wish list
   (workaround: killing of java PyCharm process).
 """
 from abc import abstractmethod, ABC
+import ast
 import os
 import pathlib
 from typing import Any, Dict, List, Optional, Sequence
 
-from ae.inspector import try_eval                                               # type: ignore
 from ae.core import DEBUG_LEVEL_VERBOSE                                         # type: ignore
 from ae.i18n import get_f_string                                                # type: ignore
 from ae.gui_app import id_of_flow, flow_action, flow_key, replace_flow_action   # type: ignore
 
 
-__version__ = '0.1.18'
+__version__ = '0.1.19'
 
 
 NODE_FILE_PREFIX = 'node_'  #: file name prefix for node imports/exports
@@ -191,6 +191,18 @@ def check_item_id(item_id: str) -> bool:
         and item_id != FLOW_PATH_ROOT_ID and FLOW_PATH_TEXT_SEP not in item_id
 
 
+def correct_item_id(item_id: str) -> str:
+    """ strip and replace extra/invalid characters from the passed item id string.
+
+    :param item_id:     item id string to correct.
+    :return:            corrected item id (can result in an empty string).
+    """
+    item_id = item_id.replace(FLOW_PATH_TEXT_SEP, '/').strip().strip('@~*-#.,;:').strip()
+    if item_id == FLOW_PATH_ROOT_ID:
+        item_id += '!'
+    return item_id
+
+
 class LiszDataMixin(ABC):
     """ lisz data model - independent from used GUI framework. """
     root_node: LiszNode             #: root of lisz data structure
@@ -249,6 +261,20 @@ class LiszDataMixin(ABC):
             self.current_node_items[0]['node'].extend(nid['node'])
         return err_msg
 
+    def add_items(self, items: LiszNode) -> str:
+        """ add item to currently displayed node.
+
+        :param items:           LiszNode list to add (each item has to have a non-empty item id).
+        :return:                error message if any error happened (multiple error messages are separated by \n),
+                                else empty string.
+        """
+        errors = list()
+        for item in items:
+            err_msg = self.add_item(item)
+            if err_msg:
+                errors.append(err_msg)
+        return "\n".join(errors)
+
     def change_sub_node_sel(self, node: LiszNode, set_sel_to: bool):
         """ change the selection of all the sub-leaves of the passed node to the specified value.
 
@@ -262,21 +288,37 @@ class LiszDataMixin(ABC):
             elif set_sel_to:
                 item['sel'] = 1.0
 
-    def current_item_or_node_repr(self) -> str:
+    @staticmethod
+    def current_item_or_node_from_literal(lit: str) -> LiszNode:
+        """ parse passed string for to integrate/add them into the current displayed node or item.
+
+        :param lit:             item/node data in string representation format.
+        :return:                LiszNode list of parsed LiszItem items.
+        """
+        if lit.startswith('['):
+            items = ast.literal_eval(lit)
+        elif lit.startswith('{'):
+            items = [ast.literal_eval(lit)]
+        else:
+            items = [dict(id=item_id) for item_id in lit.replace('\r\n', '\n').replace('\r', '\n').split('\n')
+                     if correct_item_id(item_id)]
+        return items
+
+    def current_item_or_node_literal(self) -> str:
         """ return the currently focused/displayed item or node as repr string.
 
         :return:                repr string of the currently focused item id/node or of the displayed node.
         """
         flow_id = self.flow_id
         if flow_action(flow_id) != 'focus':
-            rpr = repr(self.flow_path_node())
+            lit = repr(self.flow_path_node())
         else:
             item = self.item_by_id(flow_key(flow_id))
             if 'node' in item:
-                rpr = repr(item)
+                lit = repr(item)
             else:
-                rpr = item['id']
-        return rpr
+                lit = item['id']
+        return lit
 
     def delete_item(self, item_id: str, node_only: bool = False):
         """ delete either the complete item or the sub node of the item (identified by the passed item id).
@@ -297,7 +339,8 @@ class LiszDataMixin(ABC):
         :param old_idx:             index in the current node of the edited item or -1 if a new item (to be added).
         :param new_id:              new/edited id string.
         :param want_node:           True if the new/edited item will have a sub-node, False if not.
-        :return:                    empty string on successful validation, else error string or
+        :return:                    empty string on successful edit validation or on cancellation of new item (with
+                                    empty id string), else error string or
                                     `'request_delete_confirmation_for_item'` if the user has to confirm the deletion
                                     after the user wiped the item id string or
                                     `'request_delete_confirmation_for_node'` if the user has to confirm the
@@ -306,17 +349,22 @@ class LiszDataMixin(ABC):
         add_item = old_idx == -1
         if new_id is not None:
             if not new_id:
-                # on empty id string cancel edit if add_item, else request confirmation from user for item del
+                # on empty id cancel addition (if add_item), else request confirmation from user for item deletion
                 return "" if add_item else 'request_delete_confirmation_for_item'
-            if not check_item_id(new_id):
+
+            if add_item:
+                new_id = correct_item_id(new_id)
+            elif not check_item_id(new_id):
                 return get_f_string(
-                    "invalid item id {new_id} containing '{FLOW_PATH_TEXT_SEP}' or is equal '{FLOW_PATH_ROOT_ID}'")
+                    "invalid item id '{new_id}' containing '{FLOW_PATH_TEXT_SEP}' or is equal '{FLOW_PATH_ROOT_ID}'")
 
             new_idx = self.find_item_index(new_id)
             if new_idx != -1 and (add_item or new_idx != old_idx):
                 return get_f_string("item id '{new_id}' exists already")
 
         if add_item:                        # add new item
+            if not new_id:
+                return ""
             nid = dict(id=new_id)
             if want_node:
                 nid['node'] = list()        # type: ignore   # mypy not supports recursive types see issue #731
@@ -509,7 +557,7 @@ class LiszDataMixin(ABC):
     def import_nodes(self, file_path: str = ".") -> List[str]:
         """ import all nodes found in the documents app folder of this app into the currently displayed node.
 
-        :param file_path:   path to the folder wherein to store the node data (def=current working directory).
+        :param file_path:   path to the folder from where the node data get imported (def=current working directory).
         :return:            empty list if found node(s) got imported without errors,
                             else the list of error strings prefixed with the node file path.
         """
@@ -518,15 +566,16 @@ class LiszDataMixin(ABC):
             for node_file in os.scandir(file_path):
                 file_name = node_file.name
                 if file_name.endswith(NODE_FILE_EXT) and node_file.is_file():
+                    with open(node_file.path) as file_handle:
+                        file_content = file_handle.read()
+
                     node_name = "imported node " + file_name[len(NODE_FILE_PREFIX):-len(NODE_FILE_EXT)]
                     err_msg = self.edit_validate(-1, node_name, want_node=True)
                     if err_msg:
                         errors.append(node_file.path + ":" + err_msg)
-                        continue
-                    with open(node_file.path) as file_handle:
-                        file_content = file_handle.read()
-                    # update the list instance that got already added by edit_validate() with the loaded node data
-                    self.current_node_items[0]['node'][:] = try_eval(file_content)
+                    else:
+                        # update the list instance that got already added by edit_validate() with the loaded node data
+                        self.current_node_items[0]['node'].extend(ast.literal_eval(file_content))
         except (FileExistsError, FileNotFoundError, OSError, ValueError) as ex:
             errors.append(str(ex))
         return errors
